#!/usr/local/bin/clisp
(load "plot")

(in-package :plot)
(setq sleeptime 1)
(setq help nil)
(setf *print-length* 10)

;; first plot example
(setf rlist (make-random-list 10000))
(setf hist2 (show (histogram rlist :stats t :name "flat"
      :options '((xlabel . "random") (ylabel . "N")))))
(sleep sleeptime)

(defun sin-random ()
  "generate a random number with a sin distribution"
  (let ((x (random 3.14159)))
    (if (< (random 1.0) (sin x)) x (sin-random))))

(setf sin-dist ()) (dotimes (k 10001 sin-dist) (push (sin-random) sin-dist))
(setf hist (show (histogram sin-dist :binwidth .1 :dash t :stats t
      :options '((xlabel . "sinusoidal random") (ylabel . "N")))))
(sleep sleeptime)
(setf hist-nostats (show (histogram sin-dist :binwidth .1 :dash t
      :options '((xlabel . "sinusoidal random") (ylabel . "N")))))

(show (glist-append hist hist2))
(sleep sleeptime)
;; Now show with cdf superimposed

(setf cdf  (scatter-plot (cdf sin-dist) :color color::red :join t
           :options '((ylabel . "cdf")(yaxis . :y1) 
                      (title . "histogram with cdf"))))
(show (glist-append hist cdf))
(sleep sleeptime)

;; now plot some functions

(setf sin-plot (show (fun-plot #'sin 0 (* 2 3.1416)
                      :options '(title . "sin(x)"))))
(sleep sleeptime)
(setf sin-plot (show (fun-plot #'(lambda (x) (/ (sin x) x)) .001 30
                      :options '(title . "sin(x)/x"))))
(sleep sleeptime)

;; superimpose a bunch of function plots
(show (glist-append (mapcar #'(lambda (phi) 
   (let ((phirad (* (/ phi 180) 3.14159)))
     (fun-plot 
       #'(lambda (x) (sin (+ x phirad))) 0 7
      :color (elt color::colorlist (floor (/ phi 20)))
      :options '(title . "MultiColored Sin Curves")
      )))
   (range 0 360 20)))) 
(sleep sleeptime)

;; a bar-graph
(setf bgdt1 '(("Jan" -10) ("Feb" -5) ("Mar" 20) ("Apr" 40) ("May" 50) ("Jun" 60)
              ("Jul" 70) ("Aug" 70) ("Sep" 55) ("Oct" 40) ("Nov" 35) ("Dec" 10)))
(setf bgdt2 '(("Jan" 1) ("Feb" 5) ("Mar" 15) ("Apr" 21) ("May" 10) ("Jun" 29)
              ("Jul" 20) ("Aug" 33) ("Sep" 53) ("Oct" 36) ("Nov" 13) ("Dec" 0)))

(show (bar-graph bgdt1 :name "temperature" :fill t :barwidth .85))
(sleep sleeptime)
(setf bg1 (bar-graph bgdt1 :name " Temperature" :fill t :barwidth .4 
           :offset -0.22))

(setf bg2 (bar-graph bgdt2 :name " Rainfall" :fill t :barwidth .4 
           :offset 0.22 :color color::red))
(show (glist-append bg1 bg2))
;; now for 2 dimensions
(setf xlist (coerce (make-random-list 100001 99.9) 'vector))
(setf xlist (map 'vector #'random (map 'vector #'1+ xlist)))
(setf ab (thread rlist xlist))
(show (scatter-plot ab :symbol :smsqe :stats t
       :options '((xlabel . "random")(ylabel . "Random (random)") 
                  (title . "Scatter Plot"))))
(sleep sleeptime)
  ;; or a little bigger
(show (scatter-plot ab :symbol :smsqe :stats t
       :options '((xlabel . "random")(ylabel . "Random (random)")
                  (title . "Scatter Plot"))) '((xsize . 600)(ysize . 600)))

(sleep sleeptime)
;; how about a profile...
(setf xy (thread sin-dist 
            (mapcar #'(lambda (x) (+ (* 3 x) (* (sin-random) 8))) sin-dist)))
(setf pr (profile xy :color color::green :stats t
       :options '((xlabel . "sin dist")(ylabel . "f(x)") (yaxis . :y1)
                  (title . "Profile Plot"))))            
(show pr) (sleep sleeptime)
(setf sp (scatter-plot xy :symbol :smcr :profile t :color color::green :stats t
       :options '((xlabel . "sin dist")(ylabel . "f(x)") (yaxis . :y1)
                  (title . "Profile Scatter Plot"))))            
(show sp) (sleep sleeptime)
(show (glist-append sp hist-nostats))
(sleep sleeptime)
