#!/usr/local/bin/clisp
(load "./datoura") 

(in-package :datoura)
(setq filename "baseball.csv")
(setq sleeptime 2)
(setq help nil)

(setf dt (dtable-import filename))  ; loads the file into dtable structure
(print dt)

;; first plot example

(show (scatter-by dt '(sorate salary) '(throws) :where '(< salary 200000))) 
(sleep sleeptime)

(setf hw (get-cols dt '(height weight)))
(setf hs (get-cols dt '(height salary)))

(setf h (histogram (kill-nil (get-col dt 'sorate)) 
        :options '((xlabel . "strike out rate")(ylabel . "N"))))
(show h)
(sleep sleeptime)
(show (histogram (get-col dt 'sorate) 
        :options '((binwidth . 1)(xlabel . "strike out rate")(ylabel . "N"))))

(sleep sleeptime)
(show (histogram (get-col dt 'salary) :color color::firebrick :line t
        :options '((xlabel . "salary")(ylabel . "N"))))

(sleep sleeptime)
(show (scatter-plot (thread (first hw) (second hw)) 
    :color color::blue :symbol :mdsqe :profile t
    :options '((title . "Height vs. Weight of Major Leaguers") 
    (xlabel . "height (in)") (ylabel . "weight (lb)"))))

(sleep sleeptime)

;; An example of 2 plots superposed, with a common x axis and different y axes. 
(setf hist (histogram (dt-project dt 'sorate) 
  :color color::magenta :fill t
  :options '((xlabel . "Strikeout Rate") (ylabel . "N") (yaxis . :y1))))
(setf hist2 (histogram (dt-project dt 'sorate) :options '((yaxis . :y1))))
  
(setf cdf (scatter-plot (cdf (dt-project dt 'sorate) :normalize 1)))
(show (glist-append hist cdf hist2))

(sleep sleeptime)

(setf hhh (dt-project-by dt 'sorate '(bats) :where '(> sorate 5)))
(print-keys hhh)

(setf hl () colornum -1)
(maphash #'(lambda (key val) 
    (setf hl 
      (glist-append 
        (histogram val :name key 
                       :color (elt colorlist (incf colornum)) 
                       :line t) 
        hl))) hhh)
(show hl)
(sleep sleeptime)

(show (plot-cdf dt '(+ height (random 5.0)) :by '(bats)))
(sleep sleeptime)

;; Show the movie "A Brief History of the United States"
(load "drawmap")

(clear-state)
(setf states (dtable-import "Statehood.csv"))
(setf statehash (dt-project states 'statecode :by '(year)))
(maphash #'(lambda (k v) 
  (add-states (coerce v 'list)) 
  (show-map (first k))
  (sleep sleeptime)) 
  statehash)

(cleanup)
