;;  plots.lsp. The Data Touring Machine. Copyright (C) 2004 Steve Eichblatt
;;
;;  This program is free software; you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation; either version 2 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program; if not, write to the Free Software
;;  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;;
;;  Contact: Steve Eichblatt
;;  620 E San Salvador San Jose, CA 95112. steve@eichblatt.us

(unless (find-package :utils) (load (compile-file "utils")))
(unless (find-package :graphics) (load (compile-file "graphics")))
(unless (find-package :color) (load (compile-file "color")))
(defpackage :datoura-plot
           (:nicknames :plot)
           (:use :graphics :common-lisp :utils)
           (:export #:profile #:scatter-plot #:histogram #:bar-graph
                    #:fun-plot #:parametric-plot ))
(in-package :datoura-plot)

(defparameter *scatter-plot-options* '((join . ())))
(defparameter *fun-plot-options* '((join . t)))
(defparameter *parametric-plot-options* '((join . t)))
(defparameter *histogram-options* '())
(defparameter *profile-options* '())
(defparameter *bar-graph-options* '())
;; ------------------------------------------------------ make-plot-axes
(defun make-plot-axes (xmin ymin xmax ymax &optional (options ()))
"make-plot-axes (xmin ymin xmax ymax &optional (options ())) -- makes 
the plot axes and returns the glist"
  (let ((gl ())
        (xaxis (or (assoc-val 'xaxis options) :x0))
        (yaxis (or (assoc-val 'yaxis options) :y0)))
    (when (and xmin ymin)
      (glist-add gl (stretch-point xmin ymin 
                        `(axes . (,xaxis ,yaxis)))))
    (when (and xmax ymax)
      (glist-add gl (stretch-point xmax ymax 
                       `(axes . (,xaxis ,yaxis)))))
    (glist-add gl 
      (axis-setting xaxis :label (assoc-val 'xlabel options)
                 :major-ticks   (assoc-val 'x-major-ticks options)
                 :n-minor-ticks (assoc-val 'x-minor-ticks options)))
    (glist-add gl 
      (axis-setting yaxis :label (assoc-val 'ylabel options)
                 :major-ticks   (assoc-val 'y-major-ticks options)
                 :n-minor-ticks (assoc-val 'y-minor-ticks options)))
    (unless (equal yaxis :y0) 
      (glist-add gl (axis-setting yaxis :location .85)))
    (glist-add gl  
      (text .5 .95 (assoc-val 'title options) 
         '((text-justification . center)(text-font . "times-roman-large")
           (coords . relative))))
    (values gl xaxis yaxis)))
;; ------------------------------------------------------ profile
(defun profile (seq &key binwidth (offset 0) 
                         name xmin xmax ymin ymax color options)
  "return a list of graphics objects describing the profile.
  seq is of the form: ((x1 y1) (x2 y2) ...)
  Options include ..."
  (declare (ignore offset name))
  (let* ((poptions (alist-join options *scatter-plot-options*))
        (seq (kill-nil seq 1))
        (xseq (map 'list #'(lambda (x) (elt x 0)) seq))
        (yseq (map 'list #'(lambda (x) (elt x 1)) seq))
        (xstats (simple-stats xseq)) (ystats (simple-stats yseq))
        (gl (make-glist)) (means ()) (errbars ())
        (xhash (make-hash-table)) (yhash (make-hash-table)))
    (when (or xmin xmax ymin ymax)
      (setf seq (remove-if 
                #'(lambda (p) 
                    (or (< (elt p 0) xmin) (> (elt p 0) xmax)
                        (< (elt p 1) ymin) (> (elt p 1) ymax))) seq))
      (setf xseq (map 'list #'(lambda (x) (elt x 0)) seq))
      (setf yseq (map 'list #'(lambda (x) (elt x 1)) seq)))
    (dbg :profile "xseq ~A xstats ~A" xseq xstats)             
    (setf xmin (or xmin (assoc-val 'min xstats)))
    (setf xmax (or xmax (assoc-val 'max xstats)))
    (setf ymin (or ymin (assoc-val 'min ystats)))
    (setf ymax (or ymax (assoc-val 'max ystats)))
    (setf binwidth (or binwidth (/ (calc-major-ticks xmin xmax) 2)))
    (dbg :profile "xmin ~A xmax ~A" xmin xmax)             

    (dotimes (k (length seq))
      (let* ((elem (elt seq k))
             (x (elt elem 0)) (y (elt elem 1))
             (key (floor-to-nearest x binwidth)))
        (dbg :profile "key ~A" key)             
        (push x (gethash key xhash))
        (push y (gethash key yhash))))
    (dbg :profile "xhash ~A keys ~A" xhash (sort (keys xhash) #'<))             
    (dbg :profile "yhash ~A keys ~A" yhash (sort (keys yhash) #'<))             
    (dbg :profile "color ~A" color)
    (dolist (key (sort (keys xhash) #'<))
      (let* ((xv (gethash key xhash)) (yv (gethash key yhash))
             (xst (simple-stats xv)) (yst (simple-stats yv))
             (xmean (assoc-val 'mean xst)) ;(xrms (assoc-val 'rms xst)) 
             (ymean (assoc-val 'mean yst)) (yrms (assoc-val 'rms yst))
             (rootN (sqrt (assoc-val 'N xst)))
            ; (xlo (- xmean (/ xrms rootN))) (xhi (+ xmean (/ xrms rootN)))
             (ylo (- ymean (/ yrms rootN))) (yhi (+ ymean (/ yrms rootN))))
         (push `(,xmean ,ymean) means)
         (push `((,xmean ,yhi)(,xmean ,ylo)) errbars)))
    (when color (glist-add gl (set-color color)))
    (glist-add gl (list (ps-directive "gsave 2 setlinewidth")))
    (glist-add gl (scatter-plot means :join t))
    (glist-add gl (scatter-plot means :symbol :lgpt :options poptions))
    (dbg :profile "errbars ~A" errbars)             
    (dolist (elem errbars)
      (glist-add gl (scatter-plot elem :join t)))
    (glist-add gl (ps-directive "grestore"))
    (when color (glist-add gl (unset-color)))
    gl
 )) 
;; ------------------------------------------------------ bar-graph
(defun bar-graph (seq &key (barwidth .95) (offset 0) name
                           fill color options)
"bar-graph (seq &key (barwidth .95) (offset 0) &options)
  return a list of graphics objects describing the bar-graph
  seq is of the form (('x1' yval) ('x2' yval) ...)
  Needs to do some sanity-checking!!!
  Options include min max binwidth xlabel ylabel title 
  (needs: ymax logy color fill linetype)"
  (let* ((poptions (alist-join options *bar-graph-options*))
        (data (kill-nil seq 1))
        (xseq (map 'list #'princ-to-string 
                      (map 'list #'(lambda (k) (elt k 0)) data)))
        (xdat (range (1- (length xseq))))
        (xaxis nil) (yaxis nil)
        (yseq (map 'list #'(lambda (k) (elt k 1)) data))
        (marker-label-fn #'(lambda (x xx) (declare (ignore xx))
           (if (< x (length xseq)) (elt xseq x) "")))
        (gl (make-glist))) ;( (binwidth 1))
    (dbg :bar-graph "poptions ~A" poptions)
    (dbg :bar-graph "y data ~A" yseq)
    (multiple-value-setq (gl xaxis yaxis) 
        (make-plot-axes -.5 0 (- (length xdat) .5) 0 poptions))
    (when color (glist-add gl (set-color color)))
    (dotimes (k (length xdat))
      (glist-add gl
        (rectangle `((,(+ (- (elt xdat k) (/ barwidth 2)) offset) 0) 
                         (,barwidth ,(elt yseq k)))
        `((axes . (,xaxis ,yaxis))(rec-type . ,(when fill "rectfill"))))  
        ))
    (when color (glist-add gl (unset-color)))
    (when name (glist-add gl 
      (legend-entry name :symbol :lgsq :rgbcolor color)))
    (glist-add gl
      (axis-setting xaxis :major-ticks 1 :n-minor-ticks 0 
                                :marker-label-fn marker-label-fn))
    (values gl xdat yseq)
))
 ;; ------------------------------------------------------ histogram
(defun histogram (seq &key binwidth fill color name normalize 
                      line dash options)
  "return a list of graphics objects describing the histogram
  Options include min max binwidth xlabel ylabel title 
  (needs: ymax logx logy axis color fill linetype)"
  (let* ((poptions (alist-join options *histogram-options*))
        (xstats (simple-stats seq)) (xaxis nil) (yaxis nil)
        (xmin (or (assoc-val 'min poptions) (assoc-val 'min xstats))) 
        (xmax (or (assoc-val 'max poptions) (assoc-val 'max xstats)))
        (data (remove-if-not #'numberp seq)) (frequency ())
        (gl (make-glist)))
    (dbg :histogram "poptions ~A" poptions)
    (dbg :histogram "binwidth ~A min ~A max ~A" binwidth xmin xmax)
    (setf data (remove-if #'(lambda (x) (or (< x xmin) (> x xmax))) data))
    (unless binwidth (setf binwidth (/ (calc-major-ticks xmin xmax) 2)))
    (setf frequency (frequencies data binwidth 
                     :normalize normalize))
    (dbg :histogram "binned data ~A" data)
    (dbg :histogram "frequency ~A" frequency)
    (multiple-value-setq (gl xaxis yaxis) 
        (make-plot-axes xmin 0 xmax 0 poptions))
    (when color (glist-add gl (set-color color))) 
    (case (or dash line)
       ((nil)
       (dolist (elem frequency)
        (glist-add gl
          (rectangle (list (list (first elem) 0) (list binwidth (second elem)))
          `((axes . (,xaxis ,yaxis))(rec-type . ,(when fill "rectfill"))))  
          )))
       ((t)
        (glist-add gl (ps-directive "gsave 2 setlinewidth"))
        (when dash (glist-add gl (ps-directive "[3 5] 6 setdash")))
        (glist-add gl (line 
           (mapcar #'(lambda (x) (list (+ (first x) (/ binwidth 2)) (second x)))
                        frequency) 
           `(axes . (,xaxis ,yaxis)))) 
        (glist-add gl (grestore)))
      (otherwise ()))
    (when color (glist-add gl (unset-color)))   
    (when name (glist-add gl 
       (legend-entry name :symbol :lgsq :rgbcolor color)))
    (values gl frequency)))
;; ------------------------------------------------------ fun-plot
(defun fun-plot (func xmin xmax &key dx (npoints 200) ymin ymax 
   (join t) (symbol :smpt) options)
  "return a list of graphics objects describing the function plot.
  Options include ..."
  (let* ((poptions (alist-join options *fun-plot-options*))
        (dx (or dx (/ (- xmax xmin) npoints)))
        (domain (range xmin xmax dx))
        (vals (mapcar func domain)))
     (scatter-plot (thread domain vals) :ymin ymin :ymax ymax
        :join join :symbol symbol :options poptions)))
;; ------------------------------------------------------ parametric-plot
(defun parametric-plot (xfunc yfunc tlist &key xmin xmax ymin ymax 
   (join t) (symbol :smpt) options)
  "return a list of graphics objects describing the parametric plot.
  Options include ..."
  (let* ((poptions (alist-join options *parametric-plot-options*)))
     (scatter-plot (thread (mapcar xfunc tlist) (mapcar yfunc tlist)) 
        :xmin xmin :xmax xmax :ymin ymin :ymax ymax
        :join join :symbol symbol :options poptions)))
;; ------------------------------------------------------ scatter-plot
(defun scatter-plot (lis &key color xmin xmax ymin ymax join
   (symbol :mdpt) name profile options)
  "return a list of graphics objects describing the scatter plot.
  lis is of the form: ((x1 y1) (x2 y2) ...)
  Options include ..."
  (let ((poptions (alist-join options *scatter-plot-options*))
        (lis (kill-nil lis 1))
        (xstats (simple-stats (map 'list #'first lis)))
        (ystats (simple-stats (map 'list #'second lis)))
        (gl (make-glist)) (xaxis nil) (yaxis nil))
    (setf xmin (or xmin (assoc-val 'min xstats)))
    (setf xmax (or xmax (assoc-val 'max xstats)))
    (setf ymin (or ymin (assoc-val 'min ystats))) 
    (setf ymax (or ymax (assoc-val 'max ystats)))
    (setf lis (remove-if 
                #'(lambda (p) 
                    (or (< (elt p 0) xmin) (> (elt p 0) xmax)
                        (< (elt p 1) ymin) (> (elt p 1) ymax))) lis))
    (dbg :scatter-plot "xmin ~A xmax ~A ymin ~A ymax ~A" xmin xmax ymin ymax)
    (setf xmin (floor-to-nearest xmin (calc-major-ticks xmin xmax)))
    (setf ymin (floor-to-nearest ymin (calc-major-ticks ymin ymax)))
    (setf xmax (ceiling-to-nearest xmax (calc-major-ticks xmin xmax)))
    (setf ymax (ceiling-to-nearest ymax (calc-major-ticks ymin ymax)))
    (dbg :scatter-plot "xmin ~A xmax ~A ymin ~A ymax ~A" xmin xmax ymin ymax)
    (multiple-value-setq (gl xaxis yaxis) 
        (make-plot-axes xmin ymin xmax ymax poptions))
    (when color (glist-add gl (set-color color)))
    (glist-add gl
      (cond (join (line lis `(axes . (,xaxis ,yaxis)))) 
            (t (map 'list #'(lambda (x) 
               (point (first x) (second x) 
                 `((point-symbol . ,symbol)(axes . (,xaxis ,yaxis))))) 
                  lis))))
    (when color (glist-add gl (unset-color)))
    (when profile (glist-add gl
        (profile lis :color color :options options)))
    (when name (glist-add gl 
      (legend-entry name :symbol symbol :rgbcolor color)))
    gl)) 
