
#|
(defun project-hash (hash expr)
  (let* ((args (intersection (keys hash) (arguments expr)))
         (llist (thread args
                   (loop for k in (intersection (keys hash) (arguments expr))
                     collect (gethash k hash)))))
    (eval `(project-hash-list ,llist ,expr ,args))))

(defmacro project-hash-list (llist expr args)
  `(let ,llist 
      (list ,@(loop for x in expr collect 
        `(map 'vector #'(lambda ,args ,x) ,@args)))))

(defun p-h (hash expr)
  (let* ((args (intersection (keys hash) (arguments expr)))
         (llist (thread args
                   (loop for k in (intersection (keys hash) (arguments expr))
                     collect `(gethash ',k hash)))))
    (format t "~A" llist)
    (eval `(p-h-l ,llist ,expr ,args))))

(defmacro p-h-l (llist expr args)
  
  `(let ,llist 
      (list ,@(loop for x in expr collect 
        `(map 'vector #'(lambda ,args ,x) ,@args)))))

(defmacro pht (key hash keys args expr)
  (let ((llist (thread args
         (loop for k in (intersection keys args) 
          collect `(gethash ',k ,hash)))))
  `(let ,llist 
       (list ,@(loop for x in expr collect
          `(map 'vector #'(lambda ,args ,x) ,@args)))))) 

(defun pht3 (hash expr)
  (let ((keys (keys hash))
        (args (arguments expr)))
     (format t "keys ~A" keys)
     (pht2 hash keys args expr)))

|#

(defmacro hash-project (hash expr &key names)
"hash-project (hash expr &key names) -- 
Return a hash which is the projection of expr of the columns of input hash
eg. (hash-project h (a b (+ a b))) gives a hash with 3 keys a, b, and (+ a b)
Note: should i put the loop inside the map and only pass through the data once?"
  (let* ((keys (keys (eval hash)))
         (args (intersection keys (arguments expr)))
         (llist (thread args
           (loop for k in args
          collect `(gethash ',k ,hash)))))
  `(let ,llist 
       (loadhash (thread (or ,names ',expr)
         (list ,@(loop for x in expr collect
          `(map 'vector #'(lambda ,args ,x) ,@args))))))))

(defmacro hash-project2 (hash expr &key names)
"hash-project (hash expr &key names) -- 
Return a hash which is the projection of expr of the columns of input hash
eg. (hash-project h (a b (+ a b))) gives a hash with 3 keys a, b, and (+ a b)
Note: should i put the loop inside the map and only pass through the data once?"
  `(let* ((keys (keys ,hash))
          (args (intersection keys (arguments ',expr)))
          (llist (thread args
            (loop for k in args
             collect (gethash 'k ,hash)))))
  `(let ,llist 
       (loadhash (thread (or ,names ',expr)
         (list ,@(loop for x in expr collect
          `(map 'vector #'(lambda ,args ,x) ,@args))))))))



(defmacro hash-project-where (hash expr where &key names)
"hash-project (hash expr &key names) -- 
Return a hash which is the projection of expr of the columns of input hash
eg. (hash-project h (a b (+ a b))) gives a hash with 3 keys a, b, and (+ a b)
Note: should i put the loop inside the map and only pass through the data once?"
  (let ((keys (keys (eval hash)))
         (args (intersection keys (arguments expr)))
         (llist (thread args
           (loop for k in args
          collect `(gethash ',k ,hash)))))
  `(let ,llist 
       (let ((wh `(hash-project hash where)))
       (loadhash (thread (or ,names ',expr)
         (list ,@(loop for x in expr collect
          `(extract (map 'vector #'(lambda ,args ,x) ,@args)
                (positions #'identity ,wh))))))))))



#| example:

(setf h #S(HASH-TABLE :TEST FASTHASH-EQL 
   (D . (8 9 10)) (C . (5 6 7)) (B . (3 4 5)) (A . (1 2 40))))

(project-hash h (a b (+ a b) (> a b) (* 2 (sin a) (cos b))))

gives 
(#(1 2 40) #(3 4 5) #(4 6 45) #(NIL NIL T) #(-1.6660999 -1.1887128 0.42272085))

|#


