(load "datoura")
(in-package :datoura)
(setf week '2004wk38)
(setf weeknum (subseq (symbol-name week) 6 8))

(setf vnr2fname (format nil "X:/mkaa/steve/mtg/data/vnr2_~A.csv" week))
(setf vnr4fname (format nil "X:/mkaa/steve/mtg/data/vnr4_~A.csv" week))
(setf vnr6fname (format nil "X:/mkaa/steve/mtg/data/vnr6_~A.csv" week))

(ext:shell (format nil "cp ~A vnr2_~A.csv" vnr2fname week))
(ext:shell (format nil "cp ~A vnr4_~A.csv" vnr4fname week))
(ext:shell (format nil "cp ~A vnr6_~A.csv" vnr6fname week))

(setf vnr2 
  (dtable-import (format nil "vnr2_~A.csv" week)))
(setf vnr4 
  (dtable-import (format nil "vnr4_~A.csv" week)))
(setf vnr6 
  (dtable-import (format nil "vnr6_~A.csv" week)))

(defmacro sum (lst) `(apply #'+ ,lst))

(setf head-ops '("430" "450" "640" "650" "750" "871" "875" "87A" "811"))
(setf op-codes '(("430" . "SER") ("450" . "OverWrite") ("640" . "Disk/Head")
               ("650" . "Squeeze") ("750" . "Squeeze") ("871" . "LoTemp OW")
               ("875" . "875") ("87A" . "LoTemp SER") ("811" . "PBld")))
(setf op-names (mapcar #'cdr op-codes))
(setf head-fraction 
             '(("430" . .75) ("450" . .75) ("640" . .75)
               ("650" . .75) ("750" . .75) ("871" . .75)
               ("875" . .75) ("87A" . .75) ("811" . 0)))


(defun extract-losses (model)
  (map 'list #'(lambda (x) 
     (dt-project model week :where 
        `(equalp (extract (princ-to-string oper) (range 2)) ,x))) 
     head-ops))

(defun sum-losses (model)
  (mapcar #'(lambda (x) (sum (kill-nil (coerce x 'list))))
              (extract-losses model)))

(setf v2loss (sum-losses vnr2))
(setf v4loss (sum-losses vnr4))
(setf v6loss (sum-losses vnr6))

(defun associate (data)
  (mapcar #'(lambda (x) 
     (list (assoc (first x) op-codes :test #'equalp) (second x))) 
  data))

(defun head-related-rate (data)
  (mapcar #'(lambda (x)
    (* (cdr (assoc (first x) head-fraction :test #'equalp)) (second x))) 
   data))

(setf v2loss-hr (head-related-rate (thread head-ops v2loss)))
(setf v4loss-hr (head-related-rate (thread head-ops v4loss)))
(setf v6loss-hr (head-related-rate (thread head-ops v6loss)))

(format t "~&vnr2 losses: ~A ~&Total Loss ~6,2F Yield ~6,2F ~%~%" 
    (associate (thread head-ops v2loss)) (sum v2loss) 
    (- 100 (sum v2loss)))
(format t "~&vnr4 losses: ~A ~&Total Loss ~6,2F Yield ~6,2F ~%~%" 
    (associate (thread head-ops v4loss)) (sum v4loss) 
    (- 100 (sum v4loss)))
(format t "~&vnr6 losses: ~A ~&Total Loss ~6,2F Yield ~6,2F ~%~%" 
    (associate (thread head-ops v6loss)) (sum v6loss) 
    (- 100 (sum v6loss)))

(setf v2bg (bar-graph (thread op-names v2loss-hr) :barwidth .2 :offset -.2 
            :color '(.8 .2 .2) :fill t ))

(setf v6bg (bar-graph (thread op-names v6loss-hr) :barwidth .2 :offset .2 
            :color '(.2 .2 .8) :fill t ))

(setf v4bg (bar-graph (thread op-names v4loss-hr) :barwidth .2 :offset 0 
            :color '(.2 .8 .2) :fill t 
            :options `((xlabel . "Failure Mode")(ylabel . "Fallout (%)")
           (title . 
  ,(format nil "Head Related Fallout for Van 3 by Model -- Week ~D" weeknum)))))

(setf lgnd (make-glist))
(setf a (legend-entry "VNR2" :symbol :mdsq :rgbcolor '(.8 .2 .2)))
(glist-add lgnd a)
(glist-add lgnd (legend-entry "VNR4" :symbol :mdsq :rgbcolor '(.2 .8 .2)))
(glist-add lgnd (legend-entry "VNR6" :symbol :mdsq :rgbcolor '(.2 .2 .8)))

#|
(setf lgnd (list (ps-directive ".8 .2 .2 setrgbcolor") 
                 (text .8  .8 "VNR2" '((coords . relative))) 
                 (ps-directive ".2 .8 .2 setrgbcolor") 
                 (text .8 .75 "VNR4" '((coords . relative))) 
                 (ps-directive ".2 .2 .8 setrgbcolor")
                 (text .8 .7 "VNR6" '((coords . relative)))))
|#
(show (glist-append v2bg v4bg v6bg lgnd) '((xsize . 600) (leg-xpos . .8)))
